@@ -1,6 +1,10 @@
 # -*- coding: utf-8 -*-
 
 import logging
+
+from skosprovider_sqlalchemy.utils import session_factory
+
+
 log = logging.getLogger(__name__)
 
 from skosprovider.providers import VocabularyProvider
@@ -19,7 +23,6 @@ from skosprovider_sqlalchemy.models import (
     Concept as ConceptModel,
     Collection as CollectionModel,
     Label as LabelModel,
-    MatchType as MatchTypeModel,
     Visitation
 )
 
@@ -32,6 +35,7 @@ from sqlalchemy.orm.exc import (
 )
 
 from skosprovider.uri import (
+    DefaultUrnGenerator,
     DefaultConceptSchemeUrnGenerator
 )
 
@@ -55,7 +59,7 @@ class SQLAlchemyProvider(VocabularyProvider):
       Actually creating the data in this table needs to be scheduled.
     '''
 
-    def __init__(self, metadata, session, **kwargs):
+    def __init__(self, metadata, session_maker, **kwargs):
         '''
         Create a new provider
 
@@ -64,12 +68,22 @@ class SQLAlchemyProvider(VocabularyProvider):
         :param :class:`sqlachemy.orm.session.Session` session: The database
         session.
         '''
-        super(SQLAlchemyProvider, self).__init__(metadata, **kwargs)
-        self.session = session
-        self.conceptscheme_id = metadata.get(
-            'conceptscheme_id', metadata.get('id')
-        )
-        self.concept_scheme = self._get_concept_scheme(self.conceptscheme_id)
+        if not 'subject' in metadata:
+            metadata['subject'] = []
+        self.metadata = metadata
+        if 'uri_generator' in kwargs:
+            self.uri_generator = kwargs.get('uri_generator')
+        else:
+            self.uri_generator = DefaultUrnGenerator(self.metadata.get('id'))
+        self.session_maker = session_maker
+        try:
+            self.conceptscheme_id = int(metadata.get(
+                'conceptscheme_id', metadata.get('id')
+            ))
+        except ValueError:
+            raise ValueError(
+                'Please provide a valid integer for the conceptscheme_id.'
+            )
         if 'expand_strategy' in kwargs:
             if kwargs['expand_strategy'] in ['recurse', 'visit']:
                 self.expand_strategy = kwargs['expand_strategy']
@@ -78,26 +92,32 @@ class SQLAlchemyProvider(VocabularyProvider):
                     'Unknown expand strategy.'
                 )
 
-    def _get_concept_scheme(self, id):
+    @property
+    def concept_scheme(self):
+        return self._get_concept_scheme()
+
+    @session_factory('session_maker')
+    def _get_concept_scheme(self):
         '''
-        Find a :class:`skosprovider.skos.ConceptScheme` for a certain id.
+        Find a :class:`skosprovider.skos.ConceptScheme` for this provider.
 
         :param id: Id of a conceptscheme.
         :rtype: :class:`skosprovider.skos.ConceptScheme`
         '''
         csm = self.session\
                   .query(ConceptSchemeModel)\
-                  .get(id)
-        if csm:
-            return ConceptScheme(
-                uri=csm.uri
-            )
-        else:
-            return ConceptScheme(
-                uri=DefaultConceptSchemeUrnGenerator().generate(
-                    id=self.conceptscheme_id
-                )
-            )
+                  .get(self.conceptscheme_id)
+        return ConceptScheme(
+            uri=csm.uri,
+            labels=[
+                Label(l.label, l.labeltype_id, l.language_id)
+                for l in csm.labels
+            ],
+            notes=[
+                Note(n.note, n.notetype_id, n.language_id)
+                for n in csm.notes
+            ]
+        )
 
     def _from_thing(self, thing):
         '''
@@ -110,6 +130,7 @@ class SQLAlchemyProvider(VocabularyProvider):
             return Collection(
                 id=thing.concept_id,
                 uri=thing.uri if thing.uri is not None else self.uri_generator.generate(type='collection', id=thing.concept_id),
+                concept_scheme=self.concept_scheme,
                 labels=[
                     Label(l.label, l.labeltype_id, l.language_id)
                     for l in thing.labels
@@ -132,6 +153,7 @@ class SQLAlchemyProvider(VocabularyProvider):
             return Concept(
                 id=thing.concept_id,
                 uri=thing.uri if thing.uri is not None else self.uri_generator.generate(type='concept', id=thing.concept_id),
+                concept_scheme=self.concept_scheme,
                 labels=[
                     Label(l.label, l.labeltype_id, l.language_id)
                     for l in thing.labels
@@ -148,6 +170,7 @@ class SQLAlchemyProvider(VocabularyProvider):
                 matches=matches
             )
 
+    @session_factory('session_maker')
     def get_by_id(self, id):
         try:
             thing = self.session\
@@ -160,6 +183,7 @@ class SQLAlchemyProvider(VocabularyProvider):
             return False
         return self._from_thing(thing)
 
+    @session_factory('session_maker')
     def get_by_uri(self, uri):
         '''Get all information on a concept or collection, based on a
         :term:`URI`.
@@ -197,6 +221,7 @@ class SQLAlchemyProvider(VocabularyProvider):
             'label': l.label if l is not None else None
         }
 
+    @session_factory('session_maker')
     def find(self, query, **kwargs):
         lan = self._get_language(**kwargs)
         q = self.session\
@@ -223,6 +248,7 @@ class SQLAlchemyProvider(VocabularyProvider):
         all = q.all()
         return [self._get_id_and_label(c, lan) for c in all]
 
+    @session_factory('session_maker')
     def get_all(self, **kwargs):
         all = self.session\
                   .query(Thing)\
@@ -232,6 +258,7 @@ class SQLAlchemyProvider(VocabularyProvider):
         lan = self._get_language(**kwargs)
         return [self._get_id_and_label(c, lan) for c in all]
 
+    @session_factory('session_maker')
     def get_top_concepts(self, **kwargs):
         top = self.session\
                   .query(ConceptModel)\
@@ -246,6 +273,7 @@ class SQLAlchemyProvider(VocabularyProvider):
     def expand_concept(self, id):
         return self.expand(id)
 
+    @session_factory('session_maker')
     def expand(self, id):
         try:
             thing = self.session\
@@ -273,6 +301,7 @@ class SQLAlchemyProvider(VocabularyProvider):
                 ret += self._expand_recurse(n)
         return list(set(ret))
 
+    @session_factory('session_maker')
     def _expand_visit(self, thing):
         if thing.type == 'collection':
             ret = []
@@ -300,6 +329,7 @@ class SQLAlchemyProvider(VocabularyProvider):
             ret = [id[0] for id in ids]
         return list(set(ret))
 
+    @session_factory('session_maker')
     def get_top_display(self, **kwargs):
         '''
         Returns all concepts or collections that form the top-level of a display
@@ -333,6 +363,7 @@ class SQLAlchemyProvider(VocabularyProvider):
         lan = self._get_language(**kwargs)
         return [self._get_id_and_label(c, lan) for c in res]
 
+    @session_factory('session_maker')
     def get_children_display(self, id, **kwargs):
         '''
         Return a list of concepts or collections that should be displayed
